@@ -6,7 +6,7 @@ from __future__ import unicode_literals
 # The *_FORMAT strings use the Django date format syntax,
 # see http://docs.djangoproject.com/en/dev/ref/templates/builtins/#date
 DATE_FORMAT = 'd F Y'
-TIME_FORMAT = 'g.i.s.A'
+TIME_FORMAT = 'g.i.A'
 # DATETIME_FORMAT =
 YEAR_MONTH_FORMAT = 'F Y'
 MONTH_DAY_FORMAT = 'j F'
