@@ -32,9 +32,15 @@ def submit_row(context):
     save_as = context['save_as']
     ctx = {
         'opts': opts,
-        'show_delete_link': not is_popup and context['has_delete_permission'] and change and context.get('show_delete', True),
+        'show_delete_link': (
+            not is_popup and context['has_delete_permission'] and
+            change and context.get('show_delete', True)
+        ),
         'show_save_as_new': not is_popup and change and save_as,
-        'show_save_and_add_another': context['has_add_permission'] and not is_popup and (not save_as or context['add']),
+        'show_save_and_add_another': (
+            context['has_add_permission'] and not is_popup and
+            (not save_as or context['add'])
+        ),
         'show_save_and_continue': not is_popup and context['has_change_permission'],
         'is_popup': is_popup,
         'show_save': True,
