@@ -1,50 +1,48 @@
-from importlib import import_module
 import inspect
 import os
 import re
-import warnings
+from importlib import import_module
 
-from django import template
 from django.apps import apps
 from django.conf import settings
 from django.contrib import admin
 from django.contrib.admin.views.decorators import staff_member_required
-from django.db import models
-from django.core.exceptions import ViewDoesNotExist
-from django.http import Http404
-from django.core import urlresolvers
 from django.contrib.admindocs import utils
-from django.utils.decorators import method_decorator
-from django.utils.deprecation import RemovedInDjango18Warning
-from django.utils._os import upath
+from django.core import urlresolvers
+from django.core.exceptions import ImproperlyConfigured, ViewDoesNotExist
+from django.db import models
+from django.http import Http404
+from django.template.base import (
+    InvalidTemplateLibrary, builtins, get_library, get_templatetags_modules,
+    libraries,
+)
+from django.template.engine import Engine
 from django.utils import six
+from django.utils._os import upath
+from django.utils.decorators import method_decorator
+from django.utils.inspect import func_has_no_args
 from django.utils.translation import ugettext as _
 from django.views.generic import TemplateView
 
 # Exclude methods starting with these strings from documentation
 MODEL_METHODS_EXCLUDE = ('_', 'add_', 'delete', 'save', 'set_')
 
-if getattr(settings, 'ADMIN_FOR', None):
-    warnings.warn('The ADMIN_FOR setting has been removed, you can remove '
-                  'this setting from your configuration.', RemovedInDjango18Warning,
-                  stacklevel=2)
-
 
 class BaseAdminDocsView(TemplateView):
     """
     Base view for admindocs views.
     """
     @method_decorator(staff_member_required)
-    def dispatch(self, *args, **kwargs):
+    def dispatch(self, request, *args, **kwargs):
         if not utils.docutils_is_available:
             # Display an error message for people without docutils
             self.template_name = 'admin_doc/missing_docutils.html'
-            return self.render_to_response(admin.site.each_context())
-        return super(BaseAdminDocsView, self).dispatch(*args, **kwargs)
+            return self.render_to_response(admin.site.each_context(request))
+        return super(BaseAdminDocsView, self).dispatch(request, *args, **kwargs)
 
     def get_context_data(self, **kwargs):
         kwargs.update({'root_path': urlresolvers.reverse('admin:index')})
-        kwargs.update(admin.site.each_context())
+        kwargs.update(admin.site.each_context(self.request))
         return super(BaseAdminDocsView, self).get_context_data(**kwargs)
 
 
@@ -67,8 +65,8 @@ class TemplateTagIndexView(BaseAdminDocsView):
         load_all_installed_template_libraries()
 
         tags = []
-        app_libs = list(six.iteritems(template.libraries))
-        builtin_libs = [(None, lib) for lib in template.builtins]
+        app_libs = list(six.iteritems(libraries))
+        builtin_libs = [(None, lib) for lib in builtins]
         for module_name, library in builtin_libs + app_libs:
             for tag_name, tag_func in library.tags.items():
                 title, body, metadata = utils.parse_docstring(tag_func.__doc__)
@@ -78,7 +76,7 @@ class TemplateTagIndexView(BaseAdminDocsView):
                     body = utils.parse_rst(body, 'tag', _('tag:') + tag_name)
                 for key in metadata:
                     metadata[key] = utils.parse_rst(metadata[key], 'tag', _('tag:') + tag_name)
-                if library in template.builtins:
+                if library in builtins:
                     tag_library = ''
                 else:
                     tag_library = module_name.split('.')[-1]
@@ -100,8 +98,8 @@ class TemplateFilterIndexView(BaseAdminDocsView):
         load_all_installed_template_libraries()
 
         filters = []
-        app_libs = list(six.iteritems(template.libraries))
-        builtin_libs = [(None, lib) for lib in template.builtins]
+        app_libs = list(six.iteritems(libraries))
+        builtin_libs = [(None, lib) for lib in builtins]
         for module_name, library in builtin_libs + app_libs:
             for filter_name, filter_func in library.filters.items():
                 title, body, metadata = utils.parse_docstring(filter_func.__doc__)
@@ -111,7 +109,7 @@ class TemplateFilterIndexView(BaseAdminDocsView):
                     body = utils.parse_rst(body, 'filter', _('filter:') + filter_name)
                 for key in metadata:
                     metadata[key] = utils.parse_rst(metadata[key], 'filter', _('filter:') + filter_name)
-                if library in template.builtins:
+                if library in builtins:
                     tag_library = ''
                 else:
                     tag_library = module_name.split('.')[-1]
@@ -185,18 +183,25 @@ class ModelDetailView(BaseAdminDocsView):
     template_name = 'admin_doc/model_detail.html'
 
     def get_context_data(self, **kwargs):
+        model_name = self.kwargs['model_name']
         # Get the model class.
         try:
             app_config = apps.get_app_config(self.kwargs['app_label'])
         except LookupError:
             raise Http404(_("App %(app_label)r not found") % self.kwargs)
         try:
-            model = app_config.get_model(self.kwargs['model_name'])
+            model = app_config.get_model(model_name)
         except LookupError:
             raise Http404(_("Model %(model_name)r not found in app %(app_label)r") % self.kwargs)
 
         opts = model._meta
 
+        title, body, metadata = utils.parse_docstring(model.__doc__)
+        if title:
+            title = utils.parse_rst(title, 'model', _('model:') + model_name)
+        if body:
+            body = utils.parse_rst(body, 'model', _('model:') + model_name)
+
         # Gather fields/field descriptions.
         fields = []
         for field in opts.fields:
@@ -226,7 +231,10 @@ class ModelDetailView(BaseAdminDocsView):
         for field in opts.many_to_many:
             data_type = field.rel.to.__name__
             app_label = field.rel.to._meta.app_label
-            verbose = _("related `%(app_label)s.%(object_name)s` objects") % {'app_label': app_label, 'object_name': data_type}
+            verbose = _("related `%(app_label)s.%(object_name)s` objects") % {
+                'app_label': app_label,
+                'object_name': data_type,
+            }
             fields.append({
                 'name': "%s.all" % field.name,
                 "data_type": 'List',
@@ -240,7 +248,7 @@ class ModelDetailView(BaseAdminDocsView):
 
         # Gather model methods.
         for func_name, func in model.__dict__.items():
-            if (inspect.isfunction(func) and len(inspect.getargspec(func)[0]) == 1):
+            if inspect.isfunction(func) and func_has_no_args(func):
                 try:
                     for exclude in MODEL_METHODS_EXCLUDE:
                         if func_name.startswith(exclude):
@@ -257,8 +265,11 @@ class ModelDetailView(BaseAdminDocsView):
                 })
 
         # Gather related objects
-        for rel in opts.get_all_related_objects() + opts.get_all_related_many_to_many_objects():
-            verbose = _("related `%(app_label)s.%(object_name)s` objects") % {'app_label': rel.opts.app_label, 'object_name': rel.opts.object_name}
+        for rel in opts.related_objects:
+            verbose = _("related `%(app_label)s.%(object_name)s` objects") % {
+                'app_label': rel.related_model._meta.app_label,
+                'object_name': rel.related_model._meta.object_name,
+            }
             accessor = rel.get_accessor_name()
             fields.append({
                 'name': "%s.all" % accessor,
@@ -272,9 +283,8 @@ class ModelDetailView(BaseAdminDocsView):
             })
         kwargs.update({
             'name': '%s.%s' % (opts.app_label, opts.object_name),
-            # Translators: %s is an object type name
-            'summary': _("Attributes on %s objects") % opts.object_name,
-            'description': model.__doc__,
+            'summary': title,
+            'description': body,
             'fields': fields,
         })
         return super(ModelDetailView, self).get_context_data(**kwargs)
@@ -286,14 +296,21 @@ class TemplateDetailView(BaseAdminDocsView):
     def get_context_data(self, **kwargs):
         template = self.kwargs['template']
         templates = []
-        for dir in settings.TEMPLATE_DIRS:
-            template_file = os.path.join(dir, template)
-            templates.append({
-                'file': template_file,
-                'exists': os.path.exists(template_file),
-                'contents': lambda: open(template_file).read() if os.path.exists(template_file) else '',
-                'order': list(settings.TEMPLATE_DIRS).index(dir),
-            })
+        try:
+            default_engine = Engine.get_default()
+        except ImproperlyConfigured:
+            # Non-trivial TEMPLATES settings aren't supported (#24125).
+            pass
+        else:
+            # This doesn't account for template loaders (#24128).
+            for index, directory in enumerate(default_engine.dirs):
+                template_file = os.path.join(directory, template)
+                templates.append({
+                    'file': template_file,
+                    'exists': os.path.exists(template_file),
+                    'contents': lambda: open(template_file).read() if os.path.exists(template_file) else '',
+                    'order': index,
+                })
         kwargs.update({
             'name': template,
             'templates': templates,
@@ -307,7 +324,7 @@ class TemplateDetailView(BaseAdminDocsView):
 
 def load_all_installed_template_libraries():
     # Load/register all template tag libraries from installed apps.
-    for module_name in template.get_templatetags_modules():
+    for module_name in get_templatetags_modules():
         mod = import_module(module_name)
         if not hasattr(mod, '__file__'):
             # e.g. packages installed as eggs
@@ -324,8 +341,8 @@ def load_all_installed_template_libraries():
         else:
             for library_name in libraries:
                 try:
-                    template.get_library(library_name)
-                except template.InvalidTemplateLibrary:
+                    get_library(library_name)
+                except InvalidTemplateLibrary:
                     pass
 
 
