@@ -1,12 +1,15 @@
 from __future__ import unicode_literals
 
-from django.contrib.gis.geos import fromstr, Point, LineString, LinearRing, Polygon
-from django.utils.functional import total_ordering
-from django.utils.safestring import mark_safe
+from django.contrib.gis.geos import (
+    LinearRing, LineString, Point, Polygon, fromstr,
+)
 from django.utils import six
 from django.utils.encoding import python_2_unicode_compatible
+from django.utils.functional import total_ordering
+from django.utils.html import html_safe
 
 
+@html_safe
 @python_2_unicode_compatible
 class GEvent(object):
     """
@@ -16,7 +19,7 @@ class GEvent(object):
     add_event() call.
 
     For more information please see the Google Maps API Reference:
-     http://code.google.com/apis/maps/documentation/reference.html#GEvent
+     https://developers.google.com/maps/documentation/javascript/reference#event
 
     Example:
 
@@ -54,9 +57,10 @@ class GEvent(object):
 
     def __str__(self):
         "Returns the parameter part of a GEvent."
-        return mark_safe('"%s", %s' % (self.event, self.action))
+        return '"%s", %s' % (self.event, self.action)
 
 
+@html_safe
 @python_2_unicode_compatible
 class GOverlayBase(object):
     def __init__(self):
@@ -72,14 +76,14 @@ class GOverlayBase(object):
 
     def __str__(self):
         "The string representation is the JavaScript API call."
-        return mark_safe('%s(%s)' % (self.__class__.__name__, self.js_params))
+        return '%s(%s)' % (self.__class__.__name__, self.js_params)
 
 
 class GPolygon(GOverlayBase):
     """
     A Python wrapper for the Google GPolygon object.  For more information
     please see the Google Maps API Reference:
-     http://code.google.com/apis/maps/documentation/reference.html#GPolygon
+     https://developers.google.com/maps/documentation/javascript/reference#Polygon
     """
     def __init__(self, poly,
                  stroke_color='#0000ff', stroke_weight=2, stroke_opacity=1,
@@ -139,7 +143,7 @@ class GPolyline(GOverlayBase):
     """
     A Python wrapper for the Google GPolyline object.  For more information
     please see the Google Maps API Reference:
-     http://code.google.com/apis/maps/documentation/reference.html#GPolyline
+     https://developers.google.com/maps/documentation/javascript/reference#Polyline
     """
     def __init__(self, geom, color='#0000ff', weight=2, opacity=1):
         """
@@ -190,7 +194,7 @@ class GIcon(object):
     in turn, correspond to a subset of the attributes of the official GIcon
     javascript object:
 
-    http://code.google.com/apis/maps/documentation/reference.html#GIcon
+    https://developers.google.com/maps/documentation/javascript/reference#Icon
 
     Because a Google map often uses several different icons, a name field has
     been added to the required arguments.
@@ -263,7 +267,7 @@ class GMarker(GOverlayBase):
     """
     A Python wrapper for the Google GMarker object.  For more information
     please see the Google Maps API Reference:
-     http://code.google.com/apis/maps/documentation/reference.html#GMarker
+     https://developers.google.com/maps/documentation/javascript/reference#Marker
 
     Example:
 
