@@ -1,7 +1,8 @@
 # -*- coding: utf-8 -*-
 from __future__ import unicode_literals
 
-from django.db import models, migrations
+import django.contrib.sessions.models
+from django.db import migrations, models
 
 
 class Migration(migrations.Migration):
@@ -23,5 +24,8 @@ class Migration(migrations.Migration):
                 'verbose_name_plural': 'sessions',
             },
             bases=(models.Model,),
+            managers=[
+                ('objects', django.contrib.sessions.models.SessionManager()),
+            ],
         ),
     ]
