@@ -1,3 +1,4 @@
+import argparse
 import warnings
 
 from django.core.management.base import BaseCommand
@@ -6,7 +7,10 @@ from django.utils.deprecation import RemovedInDjango19Warning
 
 class Command(BaseCommand):
     help = "Runs this project as a FastCGI application. Requires flup."
-    args = '[various KEY=val options, use `runfcgi help` for help]'
+
+    def add_arguments(self, parser):
+        parser.add_argument('args', nargs=argparse.REMAINDER,
+            help='Various KEY=val options.')
 
     def handle(self, *args, **options):
         warnings.warn(
