@@ -3,22 +3,24 @@ from __future__ import unicode_literals
 import collections
 import datetime
 import decimal
-from importlib import import_module
-import inspect
 import math
 import os
 import re
 import sys
 import types
+from importlib import import_module
 
 from django.apps import apps
-from django.db import models, migrations
+from django.db import migrations, models
 from django.db.migrations.loader import MigrationLoader
+from django.db.migrations.operations.base import Operation
 from django.utils import datetime_safe, six
+from django.utils._os import upath
 from django.utils.encoding import force_text
 from django.utils.functional import Promise
+from django.utils.inspect import get_func_args
 from django.utils.timezone import utc
-
+from django.utils.version import get_docs_version
 
 COMPILED_REGEX_TYPE = type(re.compile(''))
 
@@ -38,17 +40,64 @@ class SettingsReference(str):
 
 
 class OperationWriter(object):
-    indentation = 2
-
-    def __init__(self, operation):
+    def __init__(self, operation, indentation=2):
         self.operation = operation
         self.buff = []
+        self.indentation = indentation
 
     def serialize(self):
+
+        def _write(_arg_name, _arg_value):
+            if (_arg_name in self.operation.serialization_expand_args and
+                    isinstance(_arg_value, (list, tuple, dict))):
+                if isinstance(_arg_value, dict):
+                    self.feed('%s={' % _arg_name)
+                    self.indent()
+                    for key, value in _arg_value.items():
+                        key_string, key_imports = MigrationWriter.serialize(key)
+                        arg_string, arg_imports = MigrationWriter.serialize(value)
+                        args = arg_string.splitlines()
+                        if len(args) > 1:
+                            self.feed('%s: %s' % (key_string, args[0]))
+                            for arg in args[1:-1]:
+                                self.feed(arg)
+                            self.feed('%s,' % args[-1])
+                        else:
+                            self.feed('%s: %s,' % (key_string, arg_string))
+                        imports.update(key_imports)
+                        imports.update(arg_imports)
+                    self.unindent()
+                    self.feed('},')
+                else:
+                    self.feed('%s=[' % _arg_name)
+                    self.indent()
+                    for item in _arg_value:
+                        arg_string, arg_imports = MigrationWriter.serialize(item)
+                        args = arg_string.splitlines()
+                        if len(args) > 1:
+                            for arg in args[:-1]:
+                                self.feed(arg)
+                            self.feed('%s,' % args[-1])
+                        else:
+                            self.feed('%s,' % arg_string)
+                        imports.update(arg_imports)
+                    self.unindent()
+                    self.feed('],')
+            else:
+                arg_string, arg_imports = MigrationWriter.serialize(_arg_value)
+                args = arg_string.splitlines()
+                if len(args) > 1:
+                    self.feed('%s=%s' % (_arg_name, args[0]))
+                    for arg in args[1:-1]:
+                        self.feed(arg)
+                    self.feed('%s,' % args[-1])
+                else:
+                    self.feed('%s=%s,' % (_arg_name, arg_string))
+                imports.update(arg_imports)
+
         imports = set()
         name, args, kwargs = self.operation.deconstruct()
-        argspec = inspect.getargspec(self.operation.__init__)
-        normalized_kwargs = inspect.getcallargs(self.operation.__init__, *args, **kwargs)
+        operation_args = get_func_args(self.operation.__init__)
 
         # See if this operation is in django.db.migrations. If it is,
         # We can just use the fact we already have that imported,
@@ -60,34 +109,19 @@ class OperationWriter(object):
             self.feed('%s.%s(' % (self.operation.__class__.__module__, name))
 
         self.indent()
-        for arg_name in argspec.args[1:]:
-            arg_value = normalized_kwargs[arg_name]
-            if (arg_name in self.operation.serialization_expand_args and
-                    isinstance(arg_value, (list, tuple, dict))):
-                if isinstance(arg_value, dict):
-                    self.feed('%s={' % arg_name)
-                    self.indent()
-                    for key, value in arg_value.items():
-                        key_string, key_imports = MigrationWriter.serialize(key)
-                        arg_string, arg_imports = MigrationWriter.serialize(value)
-                        self.feed('%s: %s,' % (key_string, arg_string))
-                        imports.update(key_imports)
-                        imports.update(arg_imports)
-                    self.unindent()
-                    self.feed('},')
-                else:
-                    self.feed('%s=[' % arg_name)
-                    self.indent()
-                    for item in arg_value:
-                        arg_string, arg_imports = MigrationWriter.serialize(item)
-                        self.feed('%s,' % arg_string)
-                        imports.update(arg_imports)
-                    self.unindent()
-                    self.feed('],')
-            else:
-                arg_string, arg_imports = MigrationWriter.serialize(arg_value)
-                self.feed('%s=%s,' % (arg_name, arg_string))
-                imports.update(arg_imports)
+
+        for i, arg in enumerate(args):
+            arg_value = arg
+            arg_name = operation_args[i]
+            _write(arg_name, arg_value)
+
+        i = len(args)
+        # Only iterate over remaining arguments
+        for arg_name in operation_args[i:]:
+            if arg_name in kwargs:  # Don't sort to maintain signature order
+                arg_value = kwargs[arg_name]
+                _write(arg_name, arg_value)
+
         self.unindent()
         self.feed('),')
         return self.render(), imports
@@ -141,7 +175,7 @@ class MigrationWriter(object):
                 imports.add("from django.conf import settings")
             else:
                 # No need to output bytestrings for dependencies
-                dependency = tuple([force_text(s) for s in dependency])
+                dependency = tuple(force_text(s) for s in dependency)
                 dependencies.append("        %s," % self.serialize(dependency)[0])
         items["dependencies"] = "\n".join(dependencies) + "\n" if dependencies else ""
 
@@ -198,7 +232,7 @@ class MigrationWriter(object):
             if not hasattr(migrations_module, '__file__'):
                 raise ImportError
 
-            basedir = os.path.dirname(migrations_module.__file__)
+            basedir = os.path.dirname(upath(migrations_module.__file__))
         except ImportError:
             app_config = apps.get_app_config(self.migration.app_label)
             migrations_package_basename = migrations_package_name.split(".")[-1]
@@ -210,11 +244,11 @@ class MigrationWriter(object):
                 # In case of using MIGRATION_MODULES setting and the custom
                 # package doesn't exist, create one.
                 package_dirs = migrations_package_name.split(".")
-                create_path = os.path.join(sys.path[0], *package_dirs)
+                create_path = os.path.join(upath(sys.path[0]), *package_dirs)
                 if not os.path.isdir(create_path):
                     os.makedirs(create_path)
                 for i in range(1, len(package_dirs) + 1):
-                    init_dir = os.path.join(sys.path[0], *package_dirs[:i])
+                    init_dir = os.path.join(upath(sys.path[0]), *package_dirs[:i])
                     init_path = os.path.join(init_dir, "__init__.py")
                     if not os.path.isfile(init_path):
                         open(init_path, "w").close()
@@ -223,13 +257,7 @@ class MigrationWriter(object):
 
     @classmethod
     def serialize_deconstructed(cls, path, args, kwargs):
-        module, name = path.rsplit(".", 1)
-        if module == "django.db.models":
-            imports = set(["from django.db import models"])
-            name = "models.%s" % name
-        else:
-            imports = set(["import %s" % module])
-            name = path
+        name, imports = cls._serialize_path(path)
         strings = []
         for arg in args:
             arg_string, arg_imports = cls.serialize(arg)
@@ -241,6 +269,17 @@ class MigrationWriter(object):
             strings.append("%s=%s" % (kw, arg_string))
         return "%s(%s)" % (name, ", ".join(strings)), imports
 
+    @classmethod
+    def _serialize_path(cls, path):
+        module, name = path.rsplit(".", 1)
+        if module == "django.db.models":
+            imports = {"from django.db import models"}
+            name = "models.%s" % name
+        else:
+            imports = {"import %s" % module}
+            name = path
+        return name, imports
+
     @classmethod
     def serialize(cls, value):
         """
@@ -264,6 +303,7 @@ class MigrationWriter(object):
                 imports.update(item_imports)
                 strings.append(item_string)
             if isinstance(value, set):
+                # Don't use the literal "{%s}" as it doesn't support empty set
                 format = "set([%s])"
             elif isinstance(value, tuple):
                 # When len(value)==0, the empty tuple should be serialized as
@@ -295,16 +335,19 @@ class MigrationWriter(object):
             value_repr = repr(value)
             if isinstance(value, datetime_safe.date):
                 value_repr = "datetime.%s" % value_repr
-            return value_repr, set(["import datetime"])
+            return value_repr, {"import datetime"}
         # Times
         elif isinstance(value, datetime.time):
             value_repr = repr(value)
             if isinstance(value, datetime_safe.time):
                 value_repr = "datetime.%s" % value_repr
             return value_repr, {"import datetime"}
+        # Timedeltas
+        elif isinstance(value, datetime.timedelta):
+            return repr(value), {"import datetime"}
         # Settings references
         elif isinstance(value, SettingsReference):
-            return "settings.%s" % value.setting_name, set(["from django.conf import settings"])
+            return "settings.%s" % value.setting_name, {"from django.conf import settings"}
         # Simple types
         elif isinstance(value, float):
             if math.isnan(value) or math.isinf(value):
@@ -326,7 +369,7 @@ class MigrationWriter(object):
             return value_repr, set()
         # Decimal
         elif isinstance(value, decimal.Decimal):
-            return repr(value), set(["from decimal import Decimal"])
+            return repr(value), {"from decimal import Decimal"}
         # Django fields
         elif isinstance(value, models.Field):
             attr_name, path, args, kwargs = value.deconstruct()
@@ -345,6 +388,17 @@ class MigrationWriter(object):
                     return value.__name__, set()
                 else:
                     return "%s.%s" % (module, value.__name__), {"import %s" % module}
+        elif isinstance(value, models.manager.BaseManager):
+            as_manager, manager_path, qs_path, args, kwargs = value.deconstruct()
+            if as_manager:
+                name, imports = cls._serialize_path(qs_path)
+                return "%s.as_manager()" % name, imports
+            else:
+                return cls.serialize_deconstructed(manager_path, args, kwargs)
+        elif isinstance(value, Operation):
+            string, imports = OperationWriter(value, indentation=0).serialize()
+            # Nested operation, trailing comma is handled in upper OperationWriter._write()
+            return string.rstrip(','), imports
         # Anything that knows how to deconstruct itself.
         elif hasattr(value, 'deconstruct'):
             return cls.serialize_deconstructed(*value.deconstruct())
@@ -354,7 +408,7 @@ class MigrationWriter(object):
             if getattr(value, "__self__", None) and isinstance(value.__self__, type):
                 klass = value.__self__
                 module = klass.__module__
-                return "%s.%s.%s" % (module, klass.__name__, value.__name__), set(["import %s" % module])
+                return "%s.%s.%s" % (module, klass.__name__, value.__name__), {"import %s" % module}
             # Further error checking
             if value.__name__ == '<lambda>':
                 raise ValueError("Cannot serialize function: lambda")
@@ -363,21 +417,22 @@ class MigrationWriter(object):
             # Python 3 is a lot easier, and only uses this branch if it's not local.
             if getattr(value, "__qualname__", None) and getattr(value, "__module__", None):
                 if "<" not in value.__qualname__:  # Qualname can include <locals>
-                    return "%s.%s" % (value.__module__, value.__qualname__), set(["import %s" % value.__module__])
+                    return "%s.%s" % (value.__module__, value.__qualname__), {"import %s" % value.__module__}
             # Python 2/fallback version
             module_name = value.__module__
             # Make sure it's actually there and not an unbound method
             module = import_module(module_name)
             if not hasattr(module, value.__name__):
                 raise ValueError(
-                    "Could not find function %s in %s.\nPlease note that "
-                    "due to Python 2 limitations, you cannot serialize "
-                    "unbound method functions (e.g. a method declared\n"
-                    "and used in the same class body). Please move the "
-                    "function into the main module body to use migrations.\n"
-                    "For more information, see https://docs.djangoproject.com/en/1.7/topics/migrations/#serializing-values"
-                    % (value.__name__, module_name))
-            return "%s.%s" % (module_name, value.__name__), set(["import %s" % module_name])
+                    "Could not find function %s in %s.\n"
+                    "Please note that due to Python 2 limitations, you cannot "
+                    "serialize unbound method functions (e.g. a method "
+                    "declared and used in the same class body). Please move "
+                    "the function into the main module body to use migrations.\n"
+                    "For more information, see "
+                    "https://docs.djangoproject.com/en/%s/topics/migrations/#serializing-values"
+                    % (value.__name__, module_name, get_docs_version()))
+            return "%s.%s" % (module_name, value.__name__), {"import %s" % module_name}
         # Other iterables
         elif isinstance(value, collections.Iterable):
             imports = set()
@@ -392,7 +447,7 @@ class MigrationWriter(object):
             return format % (", ".join(strings)), imports
         # Compiled regex
         elif isinstance(value, COMPILED_REGEX_TYPE):
-            imports = set(["import re"])
+            imports = {"import re"}
             regex_pattern, pattern_imports = cls.serialize(value.pattern)
             regex_flags, flag_imports = cls.serialize(value.flags)
             imports.update(pattern_imports)
@@ -405,8 +460,8 @@ class MigrationWriter(object):
         else:
             raise ValueError(
                 "Cannot serialize: %r\nThere are some values Django cannot serialize into "
-                "migration files.\nFor more, see https://docs.djangoproject.com/en/dev/"
-                "topics/migrations/#migration-serializing" % value
+                "migration files.\nFor more, see https://docs.djangoproject.com/en/%s/"
+                "topics/migrations/#migration-serializing" % (value, get_docs_version())
             )
 
 
@@ -414,7 +469,7 @@ MIGRATION_TEMPLATE = """\
 # -*- coding: utf-8 -*-
 from __future__ import unicode_literals
 
-from django.db import models, migrations
+from django.db import migrations, models
 %(imports)s
 
 class Migration(migrations.Migration):
