@@ -2,11 +2,14 @@
 Wrapper for loading templates from the filesystem.
 """
 
-from django.conf import settings
+import io
+
+from django.core.exceptions import SuspiciousFileOperation
 from django.template.base import TemplateDoesNotExist
-from django.template.loader import BaseLoader
 from django.utils._os import safe_join
 
+from .base import Loader as BaseLoader
+
 
 class Loader(BaseLoader):
     is_usable = True
@@ -18,30 +21,27 @@ class Loader(BaseLoader):
         template dirs are excluded from the result set, for security reasons.
         """
         if not template_dirs:
-            template_dirs = settings.TEMPLATE_DIRS
+            template_dirs = self.engine.dirs
         for template_dir in template_dirs:
             try:
                 yield safe_join(template_dir, template_name)
-            except UnicodeDecodeError:
-                # The template dir name was a bytestring that wasn't valid UTF-8.
-                raise
-            except ValueError:
-                # The joined path was located outside of this particular
-                # template_dir (it might be inside another one, so this isn't
-                # fatal).
+            except SuspiciousFileOperation:
+                # The joined path was located outside of this template_dir
+                # (it might be inside another one, so this isn't fatal).
                 pass
 
     def load_template_source(self, template_name, template_dirs=None):
         tried = []
         for filepath in self.get_template_sources(template_name, template_dirs):
             try:
-                with open(filepath, 'rb') as fp:
-                    return (fp.read().decode(settings.FILE_CHARSET), filepath)
+                with io.open(filepath, encoding=self.engine.file_charset) as fp:
+                    return fp.read(), filepath
             except IOError:
                 tried.append(filepath)
         if tried:
             error_msg = "Tried %s" % tried
         else:
-            error_msg = "Your TEMPLATE_DIRS setting is empty. Change it to point to at least one template directory."
+            error_msg = ("Your template directories configuration is empty. "
+                         "Change it to point to at least one template directory.")
         raise TemplateDoesNotExist(error_msg)
     load_template_source.is_usable = True
