@@ -4,7 +4,16 @@ Common checksum routines.
 
 __all__ = ['luhn']
 
+import warnings
+
 from django.utils import six
+from django.utils.deprecation import RemovedInDjango110Warning
+
+warnings.warn(
+    "django.utils.checksums will be removed in Django 1.10. The "
+    "luhn() function is now included in django-localflavor 1.1+.",
+    RemovedInDjango110Warning
+)
 
 LUHN_ODD_LOOKUP = (0, 2, 4, 6, 8, 1, 3, 5, 7, 9)  # sum_of_digits(index * 2)
 
