@@ -12,16 +12,18 @@ Usage:
 """
 from __future__ import unicode_literals
 
-import re
-import time
 import calendar
 import datetime
+import re
+import time
 
-from django.utils.dates import MONTHS, MONTHS_3, MONTHS_ALT, MONTHS_AP, WEEKDAYS, WEEKDAYS_ABBR
-from django.utils.translation import ugettext as _
-from django.utils.encoding import force_text
 from django.utils import six
+from django.utils.dates import (
+    MONTHS, MONTHS_3, MONTHS_ALT, MONTHS_AP, WEEKDAYS, WEEKDAYS_ABBR,
+)
+from django.utils.encoding import force_text
 from django.utils.timezone import get_default_timezone, is_aware, is_naive
+from django.utils.translation import ugettext as _
 
 re_formatchars = re.compile(r'(?<!\\)([aAbBcdDeEfFgGhHiIjlLmMnNoOPrsStTUuwWyYzZ])')
 re_escaped = re.compile(r'\\(.)')
@@ -134,6 +136,8 @@ class TimeFormat(Formatter):
             return ""
 
         seconds = self.Z()
+        if seconds == "":
+            return ""
         sign = '-' if seconds < 0 else '+'
         seconds = abs(seconds)
         return "%s%02d%02d" % (sign, seconds // 3600, (seconds // 60) % 60)
@@ -165,7 +169,14 @@ class TimeFormat(Formatter):
         if not self.timezone:
             return ""
 
-        name = self.timezone.tzname(self.data) if self.timezone else None
+        name = None
+        try:
+            name = self.timezone.tzname(self.data)
+        except Exception:
+            # pytz raises AmbiguousTimeError during the autumn DST change.
+            # This happens mainly when __init__ receives a naive datetime
+            # and sets self.timezone = get_default_timezone().
+            pass
         if name is None:
             name = self.format('O')
         return six.text_type(name)
@@ -186,7 +197,14 @@ class TimeFormat(Formatter):
         if not self.timezone:
             return ""
 
-        offset = self.timezone.utcoffset(self.data)
+        try:
+            offset = self.timezone.utcoffset(self.data)
+        except Exception:
+            # pytz raises AmbiguousTimeError during the autumn DST change.
+            # This happens mainly when __init__ receives a naive datetime
+            # and sets self.timezone = get_default_timezone().
+            return ""
+
         # `offset` is a datetime.timedelta. For negative values (to the west of
         # UTC) only days can be negative (days=-1) and seconds are always
         # positive. e.g. UTC-1 -> timedelta(days=-1, seconds=82800, microseconds=0)
@@ -226,10 +244,16 @@ class DateFormat(TimeFormat):
 
     def I(self):
         "'1' if Daylight Savings Time, '0' otherwise."
-        if self.timezone and self.timezone.dst(self.data):
-            return '1'
-        else:
-            return '0'
+        try:
+            if self.timezone and self.timezone.dst(self.data):
+                return '1'
+            else:
+                return '0'
+        except Exception:
+            # pytz raises AmbiguousTimeError during the autumn DST change.
+            # This happens mainly when __init__ receives a naive datetime
+            # and sets self.timezone = get_default_timezone().
+            return ''
 
     def j(self):
         "Day of the month without leading zeros; i.e. '1' to '31'"
