@@ -1,8 +1,5 @@
-import warnings
-
 from django import http
-from django.template import (Context, RequestContext,
-                             loader, Template, TemplateDoesNotExist)
+from django.template import Context, Engine, TemplateDoesNotExist, loader
 from django.views.decorators.csrf import requires_csrf_token
 
 
@@ -19,15 +16,17 @@ def page_not_found(request, template_name='404.html'):
         request_path
             The path of the requested URL (e.g., '/app/pages/bad_page/')
     """
+    context = {'request_path': request.path}
     try:
         template = loader.get_template(template_name)
+        body = template.render(context, request)
         content_type = None             # Django will use DEFAULT_CONTENT_TYPE
     except TemplateDoesNotExist:
-        template = Template(
+        template = Engine().from_string(
             '<h1>Not Found</h1>'
             '<p>The requested URL {{ request_path }} was not found on this server.</p>')
+        body = template.render(Context(context))
         content_type = 'text/html'
-    body = template.render(RequestContext(request, {'request_path': request.path}))
     return http.HttpResponseNotFound(body, content_type=content_type)
 
 
@@ -43,7 +42,7 @@ def server_error(request, template_name='500.html'):
         template = loader.get_template(template_name)
     except TemplateDoesNotExist:
         return http.HttpResponseServerError('<h1>Server Error (500)</h1>', content_type='text/html')
-    return http.HttpResponseServerError(template.render(Context({})))
+    return http.HttpResponseServerError(template.render())
 
 
 @requires_csrf_token
@@ -58,7 +57,7 @@ def bad_request(request, template_name='400.html'):
         template = loader.get_template(template_name)
     except TemplateDoesNotExist:
         return http.HttpResponseBadRequest('<h1>Bad Request (400)</h1>', content_type='text/html')
-    return http.HttpResponseBadRequest(template.render(Context({})))
+    return http.HttpResponseBadRequest(template.render())
 
 
 # This can be called when CsrfViewMiddleware.process_view has not run,
@@ -79,13 +78,4 @@ def permission_denied(request, template_name='403.html'):
         template = loader.get_template(template_name)
     except TemplateDoesNotExist:
         return http.HttpResponseForbidden('<h1>403 Forbidden</h1>', content_type='text/html')
-    return http.HttpResponseForbidden(template.render(RequestContext(request)))
-
-
-def shortcut(request, content_type_id, object_id):
-    warnings.warn(
-        "django.views.defaults.shortcut will be removed in Django 1.8. "
-        "Import it from django.contrib.contenttypes.views instead.",
-        PendingDeprecationWarning, stacklevel=2)
-    from django.contrib.contenttypes.views import shortcut as real_shortcut
-    return real_shortcut(request, content_type_id, object_id)
+    return http.HttpResponseForbidden(template.render(request=request))
