@@ -2,18 +2,26 @@ import sys
 import unittest
 
 from django.conf import settings
-from django.contrib.sites.models import Site
 from django.contrib.admindocs import utils
 from django.contrib.admindocs.views import get_return_data_type
 from django.contrib.auth.models import User
+from django.contrib.sites.models import Site
 from django.core.urlresolvers import reverse
 from django.test import TestCase, modify_settings, override_settings
+from django.test.utils import captured_stderr
 
-from .models import Person, Company
+from .models import Company, Person
 
 
-class MiscTests(TestCase):
-    urls = 'admin_docs.urls'
+@override_settings(
+    PASSWORD_HASHERS=('django.contrib.auth.hashers.SHA1PasswordHasher',),
+    ROOT_URLCONF='admin_docs.urls')
+@modify_settings(INSTALLED_APPS={'append': 'django.contrib.admindocs'})
+class AdminDocsTestCase(TestCase):
+    pass
+
+
+class MiscTests(AdminDocsTestCase):
 
     def setUp(self):
         User.objects.create_superuser('super', None, 'secret')
@@ -31,11 +39,9 @@ class MiscTests(TestCase):
         self.client.get('/admindocs/views/')  # should not raise
 
 
-@override_settings(PASSWORD_HASHERS=('django.contrib.auth.hashers.SHA1PasswordHasher',))
 @unittest.skipUnless(utils.docutils_is_available, "no docutils installed.")
-class AdminDocViewTests(TestCase):
+class AdminDocViewTests(AdminDocsTestCase):
     fixtures = ['data.xml']
-    urls = 'admin_docs.urls'
 
     def setUp(self):
         self.client.login(username='super', password='secret')
@@ -119,51 +125,48 @@ class AdminDocViewTests(TestCase):
             utils.docutils_is_available = True
 
 
-@override_settings(PASSWORD_HASHERS=('django.contrib.auth.hashers.SHA1PasswordHasher',))
-class XViewMiddlewareTest(TestCase):
+class XViewMiddlewareTest(AdminDocsTestCase):
     fixtures = ['data.xml']
-    urls = 'admin_docs.urls'
 
     def test_xview_func(self):
         user = User.objects.get(username='super')
         response = self.client.head('/xview/func/')
-        self.assertFalse('X-View' in response)
+        self.assertNotIn('X-View', response)
         self.client.login(username='super', password='secret')
         response = self.client.head('/xview/func/')
-        self.assertTrue('X-View' in response)
+        self.assertIn('X-View', response)
         self.assertEqual(response['X-View'], 'admin_docs.views.xview')
         user.is_staff = False
         user.save()
         response = self.client.head('/xview/func/')
-        self.assertFalse('X-View' in response)
+        self.assertNotIn('X-View', response)
         user.is_staff = True
         user.is_active = False
         user.save()
         response = self.client.head('/xview/func/')
-        self.assertFalse('X-View' in response)
+        self.assertNotIn('X-View', response)
 
     def test_xview_class(self):
         user = User.objects.get(username='super')
         response = self.client.head('/xview/class/')
-        self.assertFalse('X-View' in response)
+        self.assertNotIn('X-View', response)
         self.client.login(username='super', password='secret')
         response = self.client.head('/xview/class/')
-        self.assertTrue('X-View' in response)
+        self.assertIn('X-View', response)
         self.assertEqual(response['X-View'], 'admin_docs.views.XViewClass')
         user.is_staff = False
         user.save()
         response = self.client.head('/xview/class/')
-        self.assertFalse('X-View' in response)
+        self.assertNotIn('X-View', response)
         user.is_staff = True
         user.is_active = False
         user.save()
         response = self.client.head('/xview/class/')
-        self.assertFalse('X-View' in response)
+        self.assertNotIn('X-View', response)
 
 
 @unittest.skipUnless(utils.docutils_is_available, "no docutils installed.")
-class DefaultRoleTest(TestCase):
-    urls = 'admin_docs.urls'
+class DefaultRoleTest(AdminDocsTestCase):
 
     def test_parse_rst(self):
         """
@@ -198,21 +201,18 @@ class DefaultRoleTest(TestCase):
         self.assertEqual(parts['fragment'], markup)
 
 
-@override_settings(PASSWORD_HASHERS=('django.contrib.auth.hashers.SHA1PasswordHasher',))
 @unittest.skipUnless(utils.docutils_is_available, "no docutils installed.")
-class TestModelDetailView(TestCase):
+class TestModelDetailView(AdminDocsTestCase):
     """
     Tests that various details render correctly
     """
 
     fixtures = ['data.xml']
-    urls = 'admin_docs.urls'
 
     def setUp(self):
         self.client.login(username='super', password='secret')
-        self.response = self.client.get(
-            reverse('django-admindocs-models-detail',
-                    args=['admin_docs', 'person']))
+        with captured_stderr() as self.docutils_stderr:
+            self.response = self.client.get(reverse('django-admindocs-models-detail', args=['admin_docs', 'person']))
 
     def test_method_excludes(self):
         """
@@ -280,6 +280,29 @@ class TestModelDetailView(TestCase):
             "all related %s objects" % (link % ("admin_docs.group", "admin_docs.Group"))
         )
 
+        # "raw" and "include" directives are disabled
+        self.assertContains(self.response, '<p>&quot;raw&quot; directive disabled.</p>',)
+        self.assertContains(self.response, '.. raw:: html\n    :file: admin_docs/evilfile.txt')
+        self.assertContains(self.response, '<p>&quot;include&quot; directive disabled.</p>',)
+        self.assertContains(self.response, '.. include:: admin_docs/evilfile.txt')
+        out = self.docutils_stderr.getvalue()
+        self.assertIn('"raw" directive disabled', out)
+        self.assertIn('"include" directive disabled', out)
+
+    def test_model_with_many_to_one(self):
+        link = '<a class="reference external" href="/admindocs/models/%s/">%s</a>'
+        response = self.client.get(
+            reverse('django-admindocs-models-detail', args=['admin_docs', 'company'])
+        )
+        self.assertContains(
+            response,
+            "number of related %s objects" % (link % ("admin_docs.person", "admin_docs.Person"))
+        )
+        self.assertContains(
+            response,
+            "all related %s objects" % (link % ("admin_docs.person", "admin_docs.Person"))
+        )
+
     def test_model_with_no_backward_relations_render_only_relevant_fields(self):
         """
         A model with ``related_name`` of `+` should not show backward relationship
@@ -291,3 +314,110 @@ class TestModelDetailView(TestCase):
 
         fields = response.context_data.get('fields')
         self.assertEqual(len(fields), 2)
+
+    def test_model_docstring_renders_correctly(self):
+        summary = (
+            '<h2 class="subhead"><p>Stores information about a person, related to <a class="reference external" '
+            'href="/admindocs/models/myapp.company/">myapp.Company</a>.</p></h2>'
+        )
+        subheading = '<p><strong>Notes</strong></p>'
+        body = '<p>Use <tt class="docutils literal">save_changes()</tt> when saving this object.</p>'
+        model_body = (
+            '<dl class="docutils"><dt><tt class="'
+            'docutils literal">company</tt></dt><dd>Field storing <a class="'
+            'reference external" href="/admindocs/models/myapp.company/">'
+            'myapp.Company</a> where the person works.</dd></dl>'
+        )
+        self.assertContains(self.response, 'DESCRIPTION')
+        self.assertContains(self.response, summary, html=True)
+        self.assertContains(self.response, subheading, html=True)
+        self.assertContains(self.response, body, html=True)
+        self.assertContains(self.response, model_body, html=True)
+
+
+@unittest.skipUnless(utils.docutils_is_available, "no docutils installed.")
+class TestUtils(AdminDocsTestCase):
+    """
+    This __doc__ output is required for testing. I copied this example from
+    `admindocs` documentation. (TITLE)
+
+    Display an individual :model:`myapp.MyModel`.
+
+    **Context**
+
+    ``RequestContext``
+
+    ``mymodel``
+        An instance of :model:`myapp.MyModel`.
+
+    **Template:**
+
+    :template:`myapp/my_template.html` (DESCRIPTION)
+
+    some_metadata: some data
+
+    """
+
+    def setUp(self):
+        self.docstring = self.__doc__
+
+    def test_trim_docstring(self):
+        trim_docstring_output = utils.trim_docstring(self.docstring)
+        trimmed_docstring = (
+            'This __doc__ output is required for testing. I copied this '
+            'example from\n`admindocs` documentation. (TITLE)\n\n'
+            'Display an individual :model:`myapp.MyModel`.\n\n'
+            '**Context**\n\n``RequestContext``\n\n``mymodel``\n'
+            '    An instance of :model:`myapp.MyModel`.\n\n'
+            '**Template:**\n\n:template:`myapp/my_template.html` '
+            '(DESCRIPTION)\n\nsome_metadata: some data'
+        )
+        self.assertEqual(trim_docstring_output, trimmed_docstring)
+
+    def test_parse_docstring(self):
+        title, description, metadata = utils.parse_docstring(self.docstring)
+        docstring_title = (
+            'This __doc__ output is required for testing. I copied this example from\n'
+            '`admindocs` documentation. (TITLE)'
+        )
+        docstring_description = (
+            'Display an individual :model:`myapp.MyModel`.\n\n'
+            '**Context**\n\n``RequestContext``\n\n``mymodel``\n'
+            '    An instance of :model:`myapp.MyModel`.\n\n'
+            '**Template:**\n\n:template:`myapp/my_template.html` '
+            '(DESCRIPTION)'
+        )
+        self.assertEqual(title, docstring_title)
+        self.assertEqual(description, docstring_description)
+        self.assertEqual(metadata, {'some_metadata': 'some data'})
+
+    def test_title_output(self):
+        title, description, metadata = utils.parse_docstring(self.docstring)
+        title_output = utils.parse_rst(title, 'model', 'model:admindocs')
+        self.assertIn('TITLE', title_output)
+
+        title_rendered = (
+            '<p>This __doc__ output is required for testing. I copied this '
+            'example from\n<a class="reference external" '
+            'href="/admindocs/models/admindocs/">admindocs</a> documentation. '
+            '(TITLE)</p>\n'
+        )
+        self.assertHTMLEqual(title_output, title_rendered)
+
+    def test_description_output(self):
+        title, description, metadata = utils.parse_docstring(self.docstring)
+        description_output = utils.parse_rst(description, 'model', 'model:admindocs')
+
+        description_rendered = (
+            '<p>Display an individual <a class="reference external" '
+            'href="/admindocs/models/myapp.mymodel/">myapp.MyModel</a>.</p>\n'
+            '<p><strong>Context</strong></p>\n<p><tt class="docutils literal">'
+            'RequestContext</tt></p>\n<dl class="docutils">\n<dt><tt class="'
+            'docutils literal">mymodel</tt></dt>\n<dd>An instance of <a class="'
+            'reference external" href="/admindocs/models/myapp.mymodel/">'
+            'myapp.MyModel</a>.</dd>\n</dl>\n<p><strong>Template:</strong></p>'
+            '\n<p><a class="reference external" href="/admindocs/templates/'
+            'myapp/my_template.html/">myapp/my_template.html</a> (DESCRIPTION)'
+            '</p>\n'
+        )
+        self.assertHTMLEqual(description_output, description_rendered)
