@@ -1,14 +1,17 @@
 # -*- coding: utf-8 -*-
 from __future__ import unicode_literals
+
 import logging
 
 from django.conf import settings
-from django.core.context_processors import csrf
 from django.http import HttpRequest, HttpResponse
-from django.middleware.csrf import CsrfViewMiddleware, CSRF_KEY_LENGTH
+from django.middleware.csrf import CSRF_KEY_LENGTH, CsrfViewMiddleware
 from django.template import RequestContext, Template
+from django.template.context_processors import csrf
 from django.test import TestCase, override_settings
-from django.views.decorators.csrf import csrf_exempt, requires_csrf_token, ensure_csrf_cookie
+from django.views.decorators.csrf import (
+    csrf_exempt, ensure_csrf_cookie, requires_csrf_token,
+)
 
 
 # Response/views used for CsrfResponseMiddleware and CsrfViewMiddleware tests
@@ -119,7 +122,7 @@ class CsrfViewMiddlewareTest(TestCase):
         self.assertEqual(csrf_cookie['secure'], True)
         self.assertEqual(csrf_cookie['httponly'], True)
         self.assertEqual(csrf_cookie['path'], '/test/')
-        self.assertTrue('Cookie' in resp2.get('Vary', ''))
+        self.assertIn('Cookie', resp2.get('Vary', ''))
 
     def test_process_response_get_token_not_used(self):
         """
@@ -166,7 +169,7 @@ class CsrfViewMiddlewareTest(TestCase):
         """
         req = self._get_POST_request_with_token()
         req2 = CsrfViewMiddleware().process_view(req, post_form_view, (), {})
-        self.assertEqual(None, req2)
+        self.assertIsNone(req2)
 
     def test_process_request_csrf_cookie_no_token_exempt_view(self):
         """
@@ -175,7 +178,7 @@ class CsrfViewMiddlewareTest(TestCase):
         """
         req = self._get_POST_csrf_cookie_request()
         req2 = CsrfViewMiddleware().process_view(req, csrf_exempt(post_form_view), (), {})
-        self.assertEqual(None, req2)
+        self.assertIsNone(req2)
 
     def test_csrf_token_in_header(self):
         """
@@ -184,7 +187,7 @@ class CsrfViewMiddlewareTest(TestCase):
         req = self._get_POST_csrf_cookie_request()
         req.META['HTTP_X_CSRFTOKEN'] = self._csrf_id
         req2 = CsrfViewMiddleware().process_view(req, post_form_view, (), {})
-        self.assertEqual(None, req2)
+        self.assertIsNone(req2)
 
     def test_put_and_delete_rejected(self):
         """
@@ -209,13 +212,13 @@ class CsrfViewMiddlewareTest(TestCase):
         req.method = 'PUT'
         req.META['HTTP_X_CSRFTOKEN'] = self._csrf_id
         req2 = CsrfViewMiddleware().process_view(req, post_form_view, (), {})
-        self.assertEqual(None, req2)
+        self.assertIsNone(req2)
 
         req = self._get_GET_csrf_cookie_request()
         req.method = 'DELETE'
         req.META['HTTP_X_CSRFTOKEN'] = self._csrf_id
         req2 = CsrfViewMiddleware().process_view(req, post_form_view, (), {})
-        self.assertEqual(None, req2)
+        self.assertIsNone(req2)
 
     # Tests for the template tag method
     def test_token_node_no_csrf_cookie(self):
@@ -285,7 +288,7 @@ class CsrfViewMiddlewareTest(TestCase):
         req.META['HTTP_HOST'] = 'www.example.com'
         req.META['HTTP_REFERER'] = 'https://www.evil.org/somepage'
         req2 = CsrfViewMiddleware().process_view(req, post_form_view, (), {})
-        self.assertNotEqual(None, req2)
+        self.assertIsNotNone(req2)
         self.assertEqual(403, req2.status_code)
 
     @override_settings(ALLOWED_HOSTS=['www.example.com'])
@@ -298,7 +301,12 @@ class CsrfViewMiddlewareTest(TestCase):
         req.META['HTTP_HOST'] = 'www.example.com'
         req.META['HTTP_REFERER'] = 'http://http://www.example.com/'
         req2 = CsrfViewMiddleware().process_view(req, post_form_view, (), {})
-        self.assertNotEqual(None, req2)
+        self.assertIsNotNone(req2)
+        self.assertEqual(403, req2.status_code)
+        # Non-ASCII
+        req.META['HTTP_REFERER'] = b'\xd8B\xf6I\xdf'
+        req2 = CsrfViewMiddleware().process_view(req, post_form_view, (), {})
+        self.assertIsNotNone(req2)
         self.assertEqual(403, req2.status_code)
 
     @override_settings(ALLOWED_HOSTS=['www.example.com'])
@@ -311,7 +319,7 @@ class CsrfViewMiddlewareTest(TestCase):
         req.META['HTTP_HOST'] = 'www.example.com'
         req.META['HTTP_REFERER'] = 'https://www.example.com/somepage'
         req2 = CsrfViewMiddleware().process_view(req, post_form_view, (), {})
-        self.assertEqual(None, req2)
+        self.assertIsNone(req2)
 
     @override_settings(ALLOWED_HOSTS=['www.example.com'])
     def test_https_good_referer_2(self):
@@ -325,7 +333,7 @@ class CsrfViewMiddlewareTest(TestCase):
         req.META['HTTP_HOST'] = 'www.example.com'
         req.META['HTTP_REFERER'] = 'https://www.example.com'
         req2 = CsrfViewMiddleware().process_view(req, post_form_view, (), {})
-        self.assertEqual(None, req2)
+        self.assertIsNone(req2)
 
     def test_ensures_csrf_cookie_no_middleware(self):
         """
@@ -340,7 +348,7 @@ class CsrfViewMiddlewareTest(TestCase):
         req = self._get_GET_no_csrf_cookie_request()
         resp = view(req)
         self.assertTrue(resp.cookies.get(settings.CSRF_COOKIE_NAME, False))
-        self.assertTrue('Cookie' in resp.get('Vary', ''))
+        self.assertIn('Cookie', resp.get('Vary', ''))
 
     def test_ensures_csrf_cookie_with_middleware(self):
         """
@@ -357,7 +365,7 @@ class CsrfViewMiddlewareTest(TestCase):
         resp = view(req)
         resp2 = CsrfViewMiddleware().process_response(req, resp)
         self.assertTrue(resp2.cookies.get(settings.CSRF_COOKIE_NAME, False))
-        self.assertTrue('Cookie' in resp2.get('Vary', ''))
+        self.assertIn('Cookie', resp2.get('Vary', ''))
 
     def test_ensures_csrf_cookie_no_logging(self):
         """
@@ -428,3 +436,44 @@ class CsrfViewMiddlewareTest(TestCase):
             resp2 = CsrfViewMiddleware().process_response(req, resp)
             max_age = resp2.cookies.get('csrfcookie').get('max-age')
             self.assertEqual(max_age, '')
+
+    def test_post_data_read_failure(self):
+        """
+        #20128 -- IOErrors during POST data reading should be caught and
+        treated as if the POST data wasn't there.
+        """
+        class CsrfPostRequest(HttpRequest):
+            """
+            HttpRequest that can raise an IOError when accessing POST data
+            """
+            def __init__(self, token, raise_error):
+                super(CsrfPostRequest, self).__init__()
+                self.method = 'POST'
+
+                self.raise_error = False
+                self.COOKIES[settings.CSRF_COOKIE_NAME] = token
+                self.POST['csrfmiddlewaretoken'] = token
+                self.raise_error = raise_error
+
+            def _load_post_and_files(self):
+                raise IOError('error reading input data')
+
+            def _get_post(self):
+                if self.raise_error:
+                    self._load_post_and_files()
+                return self._post
+
+            def _set_post(self, post):
+                self._post = post
+
+            POST = property(_get_post, _set_post)
+
+        token = 'ABC'
+
+        req = CsrfPostRequest(token, raise_error=False)
+        resp = CsrfViewMiddleware().process_view(req, post_form_view, (), {})
+        self.assertIsNone(resp)
+
+        req = CsrfPostRequest(token, raise_error=True)
+        resp = CsrfViewMiddleware().process_view(req, post_form_view, (), {})
+        self.assertEqual(resp.status_code, 403)
