@@ -6,8 +6,8 @@ from django.core.urlresolvers import reverse, reverse_lazy
 from django.utils.decorators import method_decorator
 from django.views import generic
 
+from .models import Artist, Author, Book, BookSigning, Page
 from .test_forms import AuthorForm, ContactForm
-from .models import Artist, Author, Book, Page, BookSigning
 
 
 class CustomTemplateView(generic.TemplateView):
@@ -57,6 +57,10 @@ class AuthorList(generic.ListView):
     queryset = Author.objects.all()
 
 
+class BookList(generic.ListView):
+    model = Book
+
+
 class CustomPaginator(Paginator):
     def __init__(self, queryset, page_size, orphans=0, allow_empty_first_page=True):
         super(CustomPaginator, self).__init__(
@@ -168,9 +172,7 @@ class SpecializedAuthorDelete(generic.DeleteView):
     queryset = Author.objects.all()
     template_name = 'generic_views/confirm_delete.html'
     context_object_name = 'thingy'
-
-    def get_success_url(self):
-        return reverse('authors_list')
+    success_url = reverse_lazy('authors_list')
 
 
 class BookConfig(object):
