@@ -6,34 +6,37 @@ import json
 import os
 import pickle
 import unittest
-import warnings
+import uuid
 
 from django.core.exceptions import SuspiciousOperation
 from django.core.serializers.json import DjangoJSONEncoder
 from django.core.signals import request_finished
 from django.db import close_old_connections
-from django.http import (QueryDict, HttpResponse, HttpResponseRedirect,
-                         HttpResponsePermanentRedirect, HttpResponseNotAllowed,
-                         HttpResponseNotModified, StreamingHttpResponse,
-                         SimpleCookie, BadHeaderError, JsonResponse,
-                         parse_cookie)
+from django.http import (
+    BadHeaderError, HttpResponse, HttpResponseNotAllowed,
+    HttpResponseNotModified, HttpResponsePermanentRedirect,
+    HttpResponseRedirect, JsonResponse, QueryDict, SimpleCookie,
+    StreamingHttpResponse, parse_cookie,
+)
 from django.test import TestCase
-from django.utils.deprecation import RemovedInDjango18Warning
-from django.utils.encoding import smart_str, force_text
-from django.utils.functional import lazy
-from django.utils._os import upath
 from django.utils import six
+from django.utils._os import upath
+from django.utils.encoding import force_text, smart_str
+from django.utils.functional import lazy
 
 lazystr = lazy(force_text, six.text_type)
 
 
 class QueryDictTests(unittest.TestCase):
+    def test_create_with_no_args(self):
+        self.assertEqual(QueryDict(), QueryDict(str('')))
+
     def test_missing_key(self):
-        q = QueryDict(str(''))
+        q = QueryDict()
         self.assertRaises(KeyError, q.__getitem__, 'foo')
 
     def test_immutability(self):
-        q = QueryDict(str(''))
+        q = QueryDict()
         self.assertRaises(AttributeError, q.__setitem__, 'something', 'bar')
         self.assertRaises(AttributeError, q.setlist, 'foo', ['bar'])
         self.assertRaises(AttributeError, q.appendlist, 'foo', ['bar'])
@@ -43,11 +46,11 @@ class QueryDictTests(unittest.TestCase):
         self.assertRaises(AttributeError, q.clear)
 
     def test_immutable_get_with_default(self):
-        q = QueryDict(str(''))
+        q = QueryDict()
         self.assertEqual(q.get('foo', 'default'), 'default')
 
     def test_immutable_basic_operations(self):
-        q = QueryDict(str(''))
+        q = QueryDict()
         self.assertEqual(q.getlist('foo'), [])
         if six.PY2:
             self.assertEqual(q.has_key('foo'), False)
@@ -77,10 +80,10 @@ class QueryDictTests(unittest.TestCase):
 
         if six.PY2:
             self.assertTrue(q.has_key('foo'))
-        self.assertTrue('foo' in q)
+        self.assertIn('foo', q)
         if six.PY2:
             self.assertFalse(q.has_key('bar'))
-        self.assertFalse('bar' in q)
+        self.assertNotIn('bar', q)
 
         self.assertEqual(list(six.iteritems(q)), [('foo', 'bar')])
         self.assertEqual(list(six.iterlists(q)), [('foo', ['bar'])])
@@ -97,30 +100,30 @@ class QueryDictTests(unittest.TestCase):
         self.assertEqual(q.urlencode(), 'foo=bar')
 
     def test_urlencode(self):
-        q = QueryDict(str(''), mutable=True)
+        q = QueryDict(mutable=True)
         q['next'] = '/a&b/'
         self.assertEqual(q.urlencode(), 'next=%2Fa%26b%2F')
         self.assertEqual(q.urlencode(safe='/'), 'next=/a%26b/')
-        q = QueryDict(str(''), mutable=True)
+        q = QueryDict(mutable=True)
         q['next'] = '/t\xebst&key/'
         self.assertEqual(q.urlencode(), 'next=%2Ft%C3%ABst%26key%2F')
         self.assertEqual(q.urlencode(safe='/'), 'next=/t%C3%ABst%26key/')
 
     def test_mutable_copy(self):
         """A copy of a QueryDict is mutable."""
-        q = QueryDict(str('')).copy()
+        q = QueryDict().copy()
         self.assertRaises(KeyError, q.__getitem__, "foo")
         q['name'] = 'john'
         self.assertEqual(q['name'], 'john')
 
     def test_mutable_delete(self):
-        q = QueryDict(str('')).copy()
+        q = QueryDict(mutable=True)
         q['name'] = 'john'
         del q['name']
-        self.assertFalse('name' in q)
+        self.assertNotIn('name', q)
 
     def test_basic_mutable_operations(self):
-        q = QueryDict(str('')).copy()
+        q = QueryDict(mutable=True)
         q['name'] = 'john'
         self.assertEqual(q.get('foo', 'default'), 'default')
         self.assertEqual(q.get('name', 'default'), 'john')
@@ -136,7 +139,7 @@ class QueryDictTests(unittest.TestCase):
         self.assertEqual(q['foo'], 'another')
         if six.PY2:
             self.assertTrue(q.has_key('foo'))
-        self.assertTrue('foo' in q)
+        self.assertIn('foo', q)
 
         self.assertListEqual(sorted(list(six.iteritems(q))),
                              [('foo', 'another'), ('name', 'john')])
@@ -212,7 +215,7 @@ class QueryDictTests(unittest.TestCase):
             self.assertEqual(q.getlist('foo'), ['bar', '\xff'])
 
     def test_pickle(self):
-        q = QueryDict(str(''))
+        q = QueryDict()
         q1 = pickle.loads(pickle.dumps(q, 2))
         self.assertEqual(q == q1, True)
         q = QueryDict(str('a=b&c=d'))
@@ -304,6 +307,9 @@ class HttpResponseTests(unittest.TestCase):
         f = 'zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz a\xcc\x88'.encode('latin-1')
         f = f.decode('utf-8')
         h['Content-Disposition'] = 'attachment; filename="%s"' % f
+        # This one is triggering http://bugs.python.org/issue20747, that is Python
+        # will itself insert a newline in the header
+        h['Content-Disposition'] = 'attachement; filename="EdelRot_Blu\u0308te (3)-0.JPG"'
 
     def test_newlines_in_headers(self):
         # Bug #10188: Do not allow newlines in headers (CR or LF)
@@ -344,17 +350,9 @@ class HttpResponseTests(unittest.TestCase):
         # test odd inputs
         r = HttpResponse()
         r.content = ['1', '2', 3, '\u079e']
-        #'\xde\x9e' == unichr(1950).encode('utf-8')
+        # '\xde\x9e' == unichr(1950).encode('utf-8')
         self.assertEqual(r.content, b'123\xde\x9e')
 
-        # with Content-Encoding header
-        r = HttpResponse()
-        r['Content-Encoding'] = 'winning'
-        r.content = [b'abc', b'def']
-        self.assertEqual(r.content, b'abcdef')
-        self.assertRaises(TypeError if six.PY3 else UnicodeEncodeError,
-                          setattr, r, 'content', ['\u079e'])
-
         # .content can safely be accessed multiple times.
         r = HttpResponse(iter(['hello', 'world']))
         self.assertEqual(r.content, r.content)
@@ -406,6 +404,15 @@ class HttpResponseTests(unittest.TestCase):
         r.write(b'def')
         self.assertEqual(r.content, b'abcdef')
 
+    def test_stream_interface(self):
+        r = HttpResponse('asdf')
+        self.assertEqual(r.getvalue(), b'asdf')
+
+        r = HttpResponse()
+        self.assertEqual(r.writable(), True)
+        r.writelines(['foo\n', 'bar\n', 'baz\n'])
+        self.assertEqual(r.content, b'foo\nbar\nbaz\n')
+
     def test_unsafe_redirect(self):
         bad_urls = [
             'data:text/html,<script>window.alert("xss")</script>',
@@ -474,6 +481,11 @@ class JsonResponseTests(TestCase):
         response = JsonResponse(['foo', 'bar'], safe=False)
         self.assertEqual(json.loads(response.content.decode()), ['foo', 'bar'])
 
+    def test_json_response_uuid(self):
+        u = uuid.uuid4()
+        response = JsonResponse(u, safe=False)
+        self.assertEqual(json.loads(response.content.decode()), str(u))
+
     def test_json_response_custom_encoder(self):
         class CustomDjangoJSONEncoder(DjangoJSONEncoder):
             def encode(self, o):
@@ -502,6 +514,14 @@ class StreamingHttpResponseTests(TestCase):
         self.assertEqual(list(r), [b'abc', b'def'])
         self.assertEqual(list(r), [])
 
+        # iterating over Unicode strings still yields bytestring chunks.
+        r.streaming_content = iter(['hello', 'café'])
+        chunks = list(r)
+        # '\xc3\xa9' == unichr(233).encode('utf-8')
+        self.assertEqual(chunks, [b'hello', b'caf\xc3\xa9'])
+        for chunk in chunks:
+            self.assertIsInstance(chunk, six.binary_type)
+
         # streaming responses don't have a `content` attribute.
         self.assertFalse(hasattr(r, 'content'))
 
@@ -536,6 +556,9 @@ class StreamingHttpResponseTests(TestCase):
         with self.assertRaises(Exception):
             r.tell()
 
+        r = StreamingHttpResponse(iter(['hello', 'world']))
+        self.assertEqual(r.getvalue(), b'helloworld')
+
 
 class FileCloseTests(TestCase):
 
@@ -561,9 +584,7 @@ class FileCloseTests(TestCase):
         file1 = open(filename)
         r = HttpResponse(file1)
         self.assertFalse(file1.closed)
-        with warnings.catch_warnings():
-            warnings.simplefilter("ignore", RemovedInDjango18Warning)
-            list(r)
+        list(r)
         self.assertFalse(file1.closed)
         r.close()
         self.assertTrue(file1.closed)
@@ -610,8 +631,8 @@ class CookieTests(unittest.TestCase):
         """
         c = SimpleCookie()
         c['test'] = "An,awkward;value"
-        self.assertTrue(";" not in c.output().rstrip(';'))  # IE compat
-        self.assertTrue("," not in c.output().rstrip(';'))  # Safari compat
+        self.assertNotIn(";", c.output().rstrip(';'))  # IE compat
+        self.assertNotIn(",", c.output().rstrip(';'))  # Safari compat
 
     def test_decode(self):
         """
@@ -620,7 +641,7 @@ class CookieTests(unittest.TestCase):
         c = SimpleCookie()
         c['test'] = "An,awkward;value"
         c2 = SimpleCookie()
-        c2.load(c.output())
+        c2.load(c.output()[12:])
         self.assertEqual(c['test'].value, c2['test'].value)
 
     def test_decode_2(self):
@@ -630,20 +651,20 @@ class CookieTests(unittest.TestCase):
         c = SimpleCookie()
         c['test'] = b"\xf0"
         c2 = SimpleCookie()
-        c2.load(c.output())
+        c2.load(c.output()[12:])
         self.assertEqual(c['test'].value, c2['test'].value)
 
     def test_nonstandard_keys(self):
         """
         Test that a single non-standard cookie name doesn't affect all cookies. Ticket #13007.
         """
-        self.assertTrue('good_cookie' in parse_cookie('good_cookie=yes;bad:cookie=yes').keys())
+        self.assertIn('good_cookie', parse_cookie('good_cookie=yes;bad:cookie=yes').keys())
 
     def test_repeated_nonstandard_keys(self):
         """
         Test that a repeated non-standard name doesn't affect all cookies. Ticket #15852
         """
-        self.assertTrue('good_cookie' in parse_cookie('a:=b; a:=c; good_cookie=yes').keys())
+        self.assertIn('good_cookie', parse_cookie('a:=b; a:=c; good_cookie=yes').keys())
 
     def test_httponly_after_load(self):
         """
@@ -667,3 +688,15 @@ class CookieTests(unittest.TestCase):
         r = HttpResponse()
         r.set_cookie("a:.b/", 1)
         self.assertEqual(len(r.cookies.bad_cookies), 1)
+
+    def test_pickle(self):
+        rawdata = 'Customer="WILE_E_COYOTE"; Path=/acme; Version=1'
+        expected_output = 'Set-Cookie: %s' % rawdata
+
+        C = SimpleCookie()
+        C.load(rawdata)
+        self.assertEqual(C.output(), expected_output)
+
+        for proto in range(pickle.HIGHEST_PROTOCOL + 1):
+            C1 = pickle.loads(pickle.dumps(C, protocol=proto))
+            self.assertEqual(C1.output(), expected_output)
