@@ -1,5 +1,5 @@
 """
-11. Relating an object to itself, many-to-one
+Relating an object to itself, many-to-one
 
 To define a many-to-one relationship between a model and itself, use
 ``ForeignKey('self')``.
@@ -22,6 +22,7 @@ class Category(models.Model):
     def __str__(self):
         return self.name
 
+
 @python_2_unicode_compatible
 class Person(models.Model):
     full_name = models.CharField(max_length=20)
