@@ -1,19 +1,24 @@
 import os
 import tempfile
-import warnings
-
-from django.core.exceptions import ImproperlyConfigured
-
-try:
-    from django.utils.image import Image
-except ImproperlyConfigured:
-    Image = None
+import uuid
 
+from django.contrib.contenttypes.fields import (
+    GenericForeignKey, GenericRelation,
+)
+from django.contrib.contenttypes.models import ContentType
 from django.core.files.storage import FileSystemStorage
 from django.db import models
-from django.db.models.fields.files import ImageFieldFile, ImageField
+from django.db.models.fields.files import ImageField, ImageFieldFile
+from django.db.models.fields.related import (
+    ForeignKey, ForeignObject, ManyToManyField, OneToOneField,
+)
 from django.utils import six
 
+try:
+    from PIL import Image
+except ImportError:
+    Image = None
+
 
 class Foo(models.Model):
     a = models.CharField(max_length=10)
@@ -26,7 +31,7 @@ def get_foo():
 
 class Bar(models.Model):
     b = models.CharField(max_length=10)
-    a = models.ForeignKey(Foo, default=get_foo)
+    a = models.ForeignKey(Foo, default=get_foo, related_name=b'bars')
 
 
 class Whiz(models.Model):
@@ -122,6 +127,14 @@ class DateTimeModel(models.Model):
     t = models.TimeField()
 
 
+class DurationModel(models.Model):
+    field = models.DurationField()
+
+
+class NullDurationModel(models.Model):
+    field = models.DurationField(null=True)
+
+
 class PrimaryKeyCharModel(models.Model):
     string = models.CharField(max_length=10, primary_key=True)
 
@@ -154,12 +167,10 @@ class VerboseNameField(models.Model):
     field9 = models.FileField("verbose field9", upload_to="unused")
     field10 = models.FilePathField("verbose field10")
     field11 = models.FloatField("verbose field11")
-    # Don't want to depend on Pillow/PIL in this test
-    #field_image = models.ImageField("verbose field")
+    # Don't want to depend on Pillow in this test
+    # field_image = models.ImageField("verbose field")
     field12 = models.IntegerField("verbose field12")
-    with warnings.catch_warnings(record=True) as w:
-        warnings.simplefilter("always")
-        field13 = models.IPAddressField("verbose field13")
+    field13 = models.IPAddressField("verbose field13")
     field14 = models.GenericIPAddressField("verbose field14", protocol="ipv4")
     field15 = models.NullBooleanField("verbose field15")
     field16 = models.PositiveIntegerField("verbose field16")
@@ -169,14 +180,30 @@ class VerboseNameField(models.Model):
     field20 = models.TextField("verbose field20")
     field21 = models.TimeField("verbose field21")
     field22 = models.URLField("verbose field22")
+    field23 = models.UUIDField("verbose field23")
+    field24 = models.DurationField("verbose field24")
 
 
-# This model isn't used in any test, just here to ensure it validates successfully.
+class GenericIPAddress(models.Model):
+    ip = models.GenericIPAddressField(null=True, protocol='ipv4')
+
+
+###############################################################################
+# These models aren't used in any test, just here to ensure they validate
+# successfully.
+
 # See ticket #16570.
 class DecimalLessThanOne(models.Model):
     d = models.DecimalField(max_digits=3, decimal_places=3)
 
 
+# See ticket #18389.
+class FieldClassAttributeModel(models.Model):
+    field_class = models.CharField
+
+###############################################################################
+
+
 class DataModel(models.Model):
     short_data = models.BinaryField(max_length=10, default=b'\x08')
     data = models.BinaryField()
@@ -191,7 +218,7 @@ class Document(models.Model):
 ###############################################################################
 # ImageField
 
-# If Pillow/PIL available, do these tests.
+# If Pillow available, do these tests.
 if Image:
     class TestImageFieldFile(ImageFieldFile):
         """
@@ -210,7 +237,7 @@ if Image:
         attr_class = TestImageFieldFile
 
     # Set up a temp directory for file storage.
-    temp_storage_dir = tempfile.mkdtemp(dir=os.environ['DJANGO_TEST_TEMP_DIR'])
+    temp_storage_dir = tempfile.mkdtemp()
     temp_storage = FileSystemStorage(temp_storage_dir)
     temp_upload_to_dir = os.path.join(temp_storage.location, 'tests')
 
@@ -285,4 +312,74 @@ if Image:
                                   height_field='headshot_height',
                                   width_field='headshot_width')
 
+
+class AllFieldsModel(models.Model):
+    big_integer = models.BigIntegerField()
+    binary = models.BinaryField()
+    boolean = models.BooleanField(default=False)
+    char = models.CharField(max_length=10)
+    csv = models.CommaSeparatedIntegerField(max_length=10)
+    date = models.DateField()
+    datetime = models.DateTimeField()
+    decimal = models.DecimalField(decimal_places=2, max_digits=2)
+    duration = models.DurationField()
+    email = models.EmailField()
+    file_path = models.FilePathField()
+    floatf = models.FloatField()
+    integer = models.IntegerField()
+    ip_address = models.IPAddressField()
+    generic_ip = models.GenericIPAddressField()
+    null_boolean = models.NullBooleanField()
+    positive_integer = models.PositiveIntegerField()
+    positive_small_integer = models.PositiveSmallIntegerField()
+    slug = models.SlugField()
+    small_integer = models.SmallIntegerField()
+    text = models.TextField()
+    time = models.TimeField()
+    url = models.URLField()
+    uuid = models.UUIDField()
+
+    fo = ForeignObject(
+        'self',
+        from_fields=['abstract_non_concrete_id'],
+        to_fields=['id'],
+        related_name='reverse'
+    )
+    fk = ForeignKey(
+        'self',
+        related_name='reverse2'
+    )
+    m2m = ManyToManyField('self')
+    oto = OneToOneField('self')
+
+    object_id = models.PositiveIntegerField()
+    content_type = models.ForeignKey(ContentType)
+    gfk = GenericForeignKey()
+    gr = GenericRelation(DataModel)
+
+
 ###############################################################################
+
+
+class UUIDModel(models.Model):
+    field = models.UUIDField()
+
+
+class NullableUUIDModel(models.Model):
+    field = models.UUIDField(blank=True, null=True)
+
+
+class PrimaryKeyUUIDModel(models.Model):
+    id = models.UUIDField(primary_key=True, default=uuid.uuid4)
+
+
+class RelatedToUUIDModel(models.Model):
+    uuid_fk = models.ForeignKey('PrimaryKeyUUIDModel')
+
+
+class UUIDChild(PrimaryKeyUUIDModel):
+    pass
+
+
+class UUIDGrandchild(UUIDChild):
+    pass
