@@ -1,11 +1,11 @@
- # -*- coding: utf-8 -*-
-from __future__ import absolute_import, unicode_literals
+# -*- coding: utf-8 -*-
+from __future__ import unicode_literals
 
 import datetime
+from unittest import skipIf
 
 from django.test import TestCase
 from django.utils import six
-from django.utils.unittest import skipIf
 
 from .models import Article, InternationalArticle
 
