@@ -3,9 +3,10 @@ from __future__ import unicode_literals
 
 from unittest import TestCase
 
-from django.template import Template, TemplateEncodingError, Context
-from django.utils.safestring import SafeData
+from django.template import Context, Template
+from django.template.base import TemplateEncodingError
 from django.utils import six
+from django.utils.safestring import SafeData
 
 
 class UnicodeTests(TestCase):
