@@ -1,18 +1,17 @@
 # -*- coding: utf-8 -*-
 from __future__ import unicode_literals
 
-from datetime import datetime
 import os
-from unittest import TestCase
-import warnings
+from datetime import datetime
 
-from django.utils import html, safestring
+from django.test import SimpleTestCase, ignore_warnings
+from django.utils import html, safestring, six
 from django.utils._os import upath
-from django.utils.deprecation import RemovedInDjango18Warning
+from django.utils.deprecation import RemovedInDjango110Warning
 from django.utils.encoding import force_text
 
 
-class TestUtilsHtml(TestCase):
+class TestUtilsHtml(SimpleTestCase):
 
     def check_output(self, function, value, output=None):
         """
@@ -44,7 +43,7 @@ class TestUtilsHtml(TestCase):
 
     def test_format_html(self):
         self.assertEqual(
-            html.format_html("{0} {1} {third} {fourth}",
+            html.format_html("{} {} {third} {fourth}",
                              "< Dangerous >",
                              html.mark_safe("<b>safe</b>"),
                              third="< dangerous again",
@@ -124,6 +123,7 @@ class TestUtilsHtml(TestCase):
         for value, output in items:
             self.check_output(f, value, output)
 
+    @ignore_warnings(category=RemovedInDjango110Warning)
     def test_strip_entities(self):
         f = html.strip_entities
         # Strings that should come out untouched.
@@ -142,31 +142,6 @@ class TestUtilsHtml(TestCase):
             for in_pattern, output in patterns:
                 self.check_output(f, in_pattern % {'entity': entity}, output)
 
-    def test_fix_ampersands(self):
-        with warnings.catch_warnings():
-            warnings.simplefilter("ignore", RemovedInDjango18Warning)
-            f = html.fix_ampersands
-            # Strings without ampersands or with ampersands already encoded.
-            values = ("a&#1;", "b", "&a;", "&amp; &x; ", "asdf")
-            patterns = (
-                ("%s", "%s"),
-                ("&%s", "&amp;%s"),
-                ("&%s&", "&amp;%s&amp;"),
-            )
-
-            for value in values:
-                for in_pattern, out_pattern in patterns:
-                    self.check_output(f, in_pattern % value, out_pattern % value)
-
-            # Strings with ampersands that need encoding.
-            items = (
-                ("&#;", "&amp;#;"),
-                ("&#875 ;", "&amp;#875 ;"),
-                ("&#4abc;", "&amp;#4abc;"),
-            )
-            for value, output in items:
-                self.check_output(f, value, output)
-
     def test_escapejs(self):
         f = html.escapejs
         items = (
@@ -179,20 +154,7 @@ class TestUtilsHtml(TestCase):
         for value, output in items:
             self.check_output(f, value, output)
 
-    def test_clean_html(self):
-        f = html.clean_html
-        items = (
-            ('<p>I <i>believe</i> in <b>semantic markup</b>!</p>', '<p>I <em>believe</em> in <strong>semantic markup</strong>!</p>'),
-            ('I escape & I don\'t <a href="#" target="_blank">target</a>', 'I escape &amp; I don\'t <a href="#" >target</a>'),
-            ('<p>I kill whitespace</p><br clear="all"><p>&nbsp;</p>', '<p>I kill whitespace</p>'),
-            # also a regression test for #7267: this used to raise an UnicodeDecodeError
-            ('<p>* foo</p><p>* bar</p>', '<ul>\n<li> foo</li><li> bar</li>\n</ul>'),
-        )
-        with warnings.catch_warnings():
-            warnings.simplefilter("ignore", RemovedInDjango18Warning)
-            for value, output in items:
-                self.check_output(f, value, output)
-
+    @ignore_warnings(category=RemovedInDjango110Warning)
     def test_remove_tags(self):
         f = html.remove_tags
         items = (
@@ -210,10 +172,79 @@ class TestUtilsHtml(TestCase):
         # Ensure that everything unsafe is quoted, !*'();:@&=+$,/?#[]~ is considered safe as per RFC
         self.assertEqual(quote('http://example.com/path/öäü/'), 'http://example.com/path/%C3%B6%C3%A4%C3%BC/')
         self.assertEqual(quote('http://example.com/%C3%B6/ä/'), 'http://example.com/%C3%B6/%C3%A4/')
-        self.assertEqual(quote('http://example.com/?x=1&y=2'), 'http://example.com/?x=1&y=2')
+        self.assertEqual(quote('http://example.com/?x=1&y=2+3&z='), 'http://example.com/?x=1&y=2+3&z=')
+        self.assertEqual(quote('http://example.com/?x=<>"\''), 'http://example.com/?x=%3C%3E%22%27')
+        self.assertEqual(quote('http://example.com/?q=http://example.com/?x=1%26q=django'),
+                         'http://example.com/?q=http%3A%2F%2Fexample.com%2F%3Fx%3D1%26q%3Ddjango')
+        self.assertEqual(quote('http://example.com/?q=http%3A%2F%2Fexample.com%2F%3Fx%3D1%26q%3Ddjango'),
+                         'http://example.com/?q=http%3A%2F%2Fexample.com%2F%3Fx%3D1%26q%3Ddjango')
 
     def test_conditional_escape(self):
         s = '<h1>interop</h1>'
         self.assertEqual(html.conditional_escape(s),
                          '&lt;h1&gt;interop&lt;/h1&gt;')
         self.assertEqual(html.conditional_escape(safestring.mark_safe(s)), s)
+
+    def test_html_safe(self):
+        @html.html_safe
+        class HtmlClass(object):
+            if six.PY2:
+                def __unicode__(self):
+                    return "<h1>I'm a html class!</h1>"
+            else:
+                def __str__(self):
+                    return "<h1>I'm a html class!</h1>"
+
+        html_obj = HtmlClass()
+        self.assertTrue(hasattr(HtmlClass, '__html__'))
+        self.assertTrue(hasattr(html_obj, '__html__'))
+        self.assertEqual(force_text(html_obj), html_obj.__html__())
+
+    def test_html_safe_subclass(self):
+        if six.PY2:
+            class BaseClass(object):
+                def __html__(self):
+                    # defines __html__ on its own
+                    return 'some html content'
+
+                def __unicode__(self):
+                    return 'some non html content'
+
+            @html.html_safe
+            class Subclass(BaseClass):
+                def __unicode__(self):
+                    # overrides __unicode__ and is marked as html_safe
+                    return 'some html safe content'
+        else:
+            class BaseClass(object):
+                def __html__(self):
+                    # defines __html__ on its own
+                    return 'some html content'
+
+                def __str__(self):
+                    return 'some non html content'
+
+            @html.html_safe
+            class Subclass(BaseClass):
+                def __str__(self):
+                    # overrides __str__ and is marked as html_safe
+                    return 'some html safe content'
+
+        subclass_obj = Subclass()
+        self.assertEqual(force_text(subclass_obj), subclass_obj.__html__())
+
+    def test_html_safe_defines_html_error(self):
+        msg = "can't apply @html_safe to HtmlClass because it defines __html__()."
+        with self.assertRaisesMessage(ValueError, msg):
+            @html.html_safe
+            class HtmlClass(object):
+                def __html__(self):
+                    return "<h1>I'm a html class!</h1>"
+
+    def test_html_safe_doesnt_define_str(self):
+        method_name = '__unicode__()' if six.PY2 else '__str__()'
+        msg = "can't apply @html_safe to HtmlClass because it doesn't define %s." % method_name
+        with self.assertRaisesMessage(ValueError, msg):
+            @html.html_safe
+            class HtmlClass(object):
+                pass
