@@ -1,10 +1,11 @@
-from datetime import datetime
+from __future__ import unicode_literals
+
 import sys
 import unittest
+from datetime import datetime
 
+from django.utils import http, six
 from django.utils.datastructures import MultiValueDict
-from django.utils import http
-from django.utils import six
 
 
 class TestUtilsHttp(unittest.TestCase):
@@ -17,6 +18,9 @@ class TestUtilsHttp(unittest.TestCase):
         self.assertTrue(http.same_origin('http://foo.com/', 'http://foo.com'))
         # With port
         self.assertTrue(http.same_origin('https://foo.com:8000', 'https://foo.com:8000/'))
+        # No port given but according to RFC6454 still the same origin
+        self.assertTrue(http.same_origin('http://foo.com', 'http://foo.com:80/'))
+        self.assertTrue(http.same_origin('https://foo.com', 'https://foo.com:443/'))
 
     def test_same_origin_false(self):
         # Different scheme
@@ -27,6 +31,9 @@ class TestUtilsHttp(unittest.TestCase):
         self.assertFalse(http.same_origin('http://foo.com', 'http://foo.com.evil.com'))
         # Different port
         self.assertFalse(http.same_origin('http://foo.com:8000', 'http://foo.com:8001'))
+        # No port given
+        self.assertFalse(http.same_origin('http://foo.com', 'http://foo.com:8000/'))
+        self.assertFalse(http.same_origin('https://foo.com', 'https://foo.com:8000/'))
 
     def test_urlencode(self):
         # 2-tuples (the norm)
@@ -45,7 +52,7 @@ class TestUtilsHttp(unittest.TestCase):
             'c=3&a=1&b=2',
             'c=3&b=2&a=1'
         ]
-        self.assertTrue(result in acceptable_results)
+        self.assertIn(result, acceptable_results)
         result = http.urlencode({'a': [1, 2]}, doseq=False)
         self.assertEqual(result, 'a=%5B%271%27%2C+%272%27%5D')
         result = http.urlencode({'a': [1, 2]}, doseq=True)
@@ -63,7 +70,7 @@ class TestUtilsHttp(unittest.TestCase):
             'name=Adrian&name=Simon&position=Developer',
             'position=Developer&name=Adrian&name=Simon'
         ]
-        self.assertTrue(result in acceptable_results)
+        self.assertIn(result, acceptable_results)
 
     def test_base36(self):
         # reciprocity works
@@ -128,6 +135,28 @@ class TestUtilsHttp(unittest.TestCase):
         decoded = http.urlsafe_base64_decode(encoded)
         self.assertEqual(bytestring, decoded)
 
+    def test_urlquote(self):
+        self.assertEqual(http.urlquote('Paris & Orl\xe9ans'),
+            'Paris%20%26%20Orl%C3%A9ans')
+        self.assertEqual(http.urlquote('Paris & Orl\xe9ans', safe="&"),
+            'Paris%20&%20Orl%C3%A9ans')
+        self.assertEqual(
+            http.urlunquote('Paris%20%26%20Orl%C3%A9ans'),
+            'Paris & Orl\xe9ans')
+        self.assertEqual(
+            http.urlunquote('Paris%20&%20Orl%C3%A9ans'),
+            'Paris & Orl\xe9ans')
+        self.assertEqual(http.urlquote_plus('Paris & Orl\xe9ans'),
+            'Paris+%26+Orl%C3%A9ans')
+        self.assertEqual(http.urlquote_plus('Paris & Orl\xe9ans', safe="&"),
+            'Paris+&+Orl%C3%A9ans')
+        self.assertEqual(
+            http.urlunquote_plus('Paris+%26+Orl%C3%A9ans'),
+            'Paris & Orl\xe9ans')
+        self.assertEqual(
+            http.urlunquote_plus('Paris+&+Orl%C3%A9ans'),
+            'Paris & Orl\xe9ans')
+
 
 class ETagProcessingTests(unittest.TestCase):
     def test_parsing(self):
@@ -140,6 +169,14 @@ class ETagProcessingTests(unittest.TestCase):
 
 
 class HttpDateProcessingTests(unittest.TestCase):
+    def test_http_date(self):
+        t = 1167616461.0
+        self.assertEqual(http.http_date(t), 'Mon, 01 Jan 2007 01:54:21 GMT')
+
+    def test_cookie_date(self):
+        t = 1167616461.0
+        self.assertEqual(http.cookie_date(t), 'Mon, 01-Jan-2007 01:54:21 GMT')
+
     def test_parsing_rfc1123(self):
         parsed = http.parse_http_date('Sun, 06 Nov 1994 08:49:37 GMT')
         self.assertEqual(datetime.utcfromtimestamp(parsed),
