@@ -1,11 +1,13 @@
 from __future__ import unicode_literals
 
-from django.template import Template, Context
+from django.template import Context, Template
 from django.test import TestCase
-from django.utils.encoding import force_text, force_bytes
+from django.utils import html, six, text
+from django.utils.encoding import force_bytes, force_text
 from django.utils.functional import lazy
-from django.utils.safestring import mark_safe, mark_for_escaping, SafeData, EscapeData
-from django.utils import six
+from django.utils.safestring import (
+    EscapeData, SafeData, mark_for_escaping, mark_safe,
+)
 
 lazystr = lazy(force_text, six.text_type)
 lazybytes = lazy(force_bytes, bytes)
@@ -90,3 +92,16 @@ class SafeStringTest(TestCase):
         s = mark_for_escaping(Obj())
 
         self.assertRenderEqual('{{ s }}', '&lt;obj&gt;', s=s)
+
+    def test_add_lazy_safe_text_and_safe_text(self):
+        s = html.escape(lazystr('a'))
+        s += mark_safe('&b')
+        self.assertRenderEqual('{{ s }}', 'a&b', s=s)
+
+        s = html.escapejs(lazystr('a'))
+        s += mark_safe('&b')
+        self.assertRenderEqual('{{ s }}', 'a&b', s=s)
+
+        s = text.slugify(lazystr('a'))
+        s += mark_safe('&b')
+        self.assertRenderEqual('{{ s }}', 'a&b', s=s)
