@@ -3,14 +3,14 @@ import datetime
 import pickle
 import unittest
 
+from django.test import override_settings
+from django.utils import timezone
+
 try:
     import pytz
 except ImportError:
     pytz = None
 
-from django.test import override_settings
-from django.utils import timezone
-
 
 if pytz is not None:
     CET = pytz.timezone("Europe/Paris")
@@ -71,6 +71,36 @@ class TimezoneTests(unittest.TestCase):
         finally:
             timezone.deactivate()
 
+    def test_override_decorator(self):
+        default = timezone.get_default_timezone()
+
+        @timezone.override(EAT)
+        def func_tz_eat():
+            self.assertIs(EAT, timezone.get_current_timezone())
+
+        @timezone.override(None)
+        def func_tz_none():
+            self.assertIs(default, timezone.get_current_timezone())
+
+        try:
+            timezone.activate(ICT)
+
+            func_tz_eat()
+            self.assertIs(ICT, timezone.get_current_timezone())
+
+            func_tz_none()
+            self.assertIs(ICT, timezone.get_current_timezone())
+
+            timezone.deactivate()
+
+            func_tz_eat()
+            self.assertIs(default, timezone.get_current_timezone())
+
+            func_tz_none()
+            self.assertIs(default, timezone.get_current_timezone())
+        finally:
+            timezone.deactivate()
+
     def test_copy(self):
         self.assertIsInstance(copy.copy(timezone.UTC()), timezone.UTC)
         self.assertIsInstance(copy.copy(timezone.LocalTimezone()), timezone.LocalTimezone)
