@@ -1,8 +1,7 @@
-from django.conf.urls import patterns, include, url
-
+from django.conf.urls import include, url
 from django.contrib import admin
 
-urlpatterns = patterns('',
+urlpatterns = [
     # This is the same as in the default project template
     url(r'^admin/', include(admin.site.urls)),
-)
+]
