@@ -1,14 +1,14 @@
-# coding: utf-8
+# -*- coding: utf-8 -*-
 from __future__ import unicode_literals
 
-from django.test import TestCase
+from django.test import TestCase, override_settings
 
 
+@override_settings(ROOT_URLCONF='view_tests.generic_urls')
 class URLHandling(TestCase):
     """
     Tests for URL handling in views and responses.
     """
-    urls = 'view_tests.generic_urls'
     redirect_target = "/%E4%B8%AD%E6%96%87/target/"
 
     def test_combining_redirect(self):
@@ -36,4 +36,3 @@ class URLHandling(TestCase):
         """
         response = self.client.get('/permanent_nonascii_redirect/')
         self.assertRedirects(response, self.redirect_target, status_code=301)
-
